﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  public abstract class DistanceBase<T> : Item, IDistance<T> {

    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected DistanceBase(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected DistanceBase(DistanceBase<T> original, Cloner cloner)
      : base(original, cloner) { }
    protected DistanceBase() { }
    #endregion

    public abstract double Get(T a, T b);
    public abstract double Get(T a, T b, double threshold);

    public IComparer<T> GetDistanceComparer(T item) {
      return new DistanceComparer(item, this);
    }

    private class DistanceComparer : IComparer<T> {
      private readonly T item;
      private readonly IDistance<T> dist;

      public DistanceComparer(T item, IDistance<T> dist) {
        this.dist = dist;
        this.item = item;
      }

      public int Compare(T x, T y) {
        return dist.Get(x, item).CompareTo(dist.Get(y, item));
      }
    }
  }
}